% trykkstt - karakteristikkmetoden: mag-sjakt-turb

% Hp=Ha-a/gA(Qp-Qa)-kQp|Qa|=0
% Hp=Hb+a/gA(Qp-Qa)+kQp|Qb|=0
% Grensebetingelser:
%   Magasin: H(0,t)=Ho
%   Ventil: H(N,t)=K*A*sqrt(2g*dH)

% konstanter
L=30+11580; 
L1=10050+30;
A1=17;
L2=540+L1;
A2=2.54;
L3=960+L2;
A3=2.01;
L4=30+L3;
A4=16;
As=16;
D1=sqrt(4*A1/3.14);
D2=sqrt(4*A2/3.14);
D3=sqrt(4*A3/3.14);
D4=sqrt(4*A4/3.13);
dx=30;
dt=0.025;
a=dx/dt;
g=9.81;
f1=0.05;
f2=0.01;
B1=a/(g*A1);
B2=a/(g*A2);
B3=a/(g*A3);
B4=a/(g*A4);
k1=f1*dx/(2*g*D1*A1^2);
k2=f2*dx/(2*g*D2*A2^2);
k3=f2*dx/(2*g*D3*A3^2);
k4=f2*dx/(2*g*D4*A4^2);
tmax=1500;
Qo=16.5;
Ho=945; %Er Ho mlt i moh eller kotehyde minus turbinkotehyde/undervann?
Qn=16.5;
K=1;
xo=dx;
to=1;

H = zeros(L,tmax);
Q = zeros(L,tmax);
Z = zeros(tmax);

%startbetingelser:
for t=1:1:tmax
    for x=xo:dx:L
        Q(x,t)=Qo;
        x=x+dx;
    end
    H(xo,t)=Ho;
    t=t+1;
end

for t=1:1:tmax
    if t<840
        K=K-0.00119;
    else
        K=0.0004;
    end    
    for x=xo:dx:L
        if x<=L1
            k=k1;
            B=B1;
        elseif x<=L2
            k=k2;
            B=B2;
        elseif x<=L3
            k=k3;
            B=B3;
        elseif x<=L4
            k=k4;
            B=B4;
        end
        if t==1
            Q(x,t)=Qo;
            if x==xo;
                H(x,t)=Ho;     
            else 
                H(x,t)=H(x-dx,t)-k*Q(x-dx,t)*abs(Q(x-dx,t));
            end
            zo=H(L1,t);
            Z(t)=zo;
            Hn=H(L,1);
        elseif x==xo
            Hb=H(x+dx,t-1);
            Qb=Q(x+dx,t-1);
            H(x,t)=Ho;
            Q(x,t)=1/B1*(H(x,t)-Hb-k1*Qb*abs(Qb))+Qb;     
        elseif x==L1
            Ha=H(x-dx,t-1);
            Hb=H(x+dx,t-1);
            Qa=Q(x-dx,t-1);
            Qb=Q(x+dx,t-1);
            Z(t)=Z(t-1)+dt*(Qa-Qb)/As;
            H(x,t)=Z(t);
            Q(x,t)=1/B2*(H(x,t)-Hb-k2*Qb*abs(Qb))+Qb;
        elseif x==L2
            Ha=H(x-dx,t-1);
            Hb=H(x+dx,t-1);
            Qa=Q(x-dx,t-1);
            Qb=Q(x+dx,t-1);
            H(x,t)=1/(1+B2/B3)*(Ha+B2/B3*(Hb+k3*Qb*abs(Qb))-B2*(Qb-Qa)-k2*Qa*abs(Qa));
            Q(x,t)=1/B3*(H(x,t)-Hb-k3*Qb*abs(Qb))+Qb;
        elseif x==L3
            Ha=H(x-dx,t-1);
            Qa=Q(x-dx,t-1);
            Hb=H(x+dx,t-1);
            Qb=Q(x+dx,t-1);
            H(x,t)=1/(1+B3/B4)*(Ha+B3/B4*(Hb+k4*Qb*abs(Qb))-B3*(Qb-Qa)-k3*Qa*abs(Qa));
            Q(x,t)=1/B4*(H(x,t)-Hb-k4*Qb*abs(Qb))+Qb;
        elseif x==L4
            Ha=H(x-dx,t-1);
            Qa=Q(x-dx,t-1);
            Cv=((K*Qn)^2)/Hn;
            Q(x,t)=-B4*Cv/2+sqrt((B4*Cv)^2+4*Cv*(Ha+B4*Qa-k4*Qa*abs(Qa)))/2;
            Qp=Q(x,t);
            H(x,t)=(Qp/(K*Qn))^2*Hn;
        elseif (x<L1) && (x>xo)
            Ha=H(x-dx,t-1);
            Qa=Q(x-dx,t-1);
            Hb=H(x+dx,t-1);
            Qb=Q(x+dx,t-1);
            H(x,t)=0.5*(Ha+Hb-B1*(Qb-Qa)+k1*(Qb*abs(Qb)-Qa*abs(Qa)));
            Q(x,t)=1/B1*(H(x,t)-Hb-k1*Qb*abs(Qb))+Qb;
        elseif (x<L2) && (x>L1)
            Ha=H(x-dx,t-1);
            Qa=Q(x-dx,t-1);
            Hb=H(x+dx,t-1);
            Qb=Q(x+dx,t-1);
            H(x,t)=0.5*(Ha+Hb-B2*(Qb-Qa)+k2*(Qb*abs(Qb)-Qa*abs(Qa)));
            Q(x,t)=1/B2*(H(x,t)-Hb-k2*Qb*abs(Qb))+Qb;
        elseif (x<L3) && (x>L2)
            Ha=H(x-dx,t-1);
            Qa=Q(x-dx,t-1);
            Hb=H(x+dx,t-1);
            Qb=Q(x+dx,t-1);
            H(x,t)=0.5*(Ha+Hb-B3*(Qb-Qa)+k3*(Qb*abs(Qb)-Qa*abs(Qa)));
            Q(x,t)=1/B3*(H(x,t)-Hb-k3*Qb*abs(Qb))+Qb;
        end
        x=x+dx;
    end
    t=t+1;
end

figure
plot(H(L,:))
figure
plot(Q(L,:))

Hny=H(L,:);
Qny=Q(L,:);
tid=0:dt:37.5-dt;
figure
plot(tid,Hny)
figure
plot(tid,Qny)


    